<?php
require_once('class_vurl.php');
require_once('config.php');
require('user_info.php');
require('forum_info.php');
require('thread_info.php');
require('page_nav.php');

class VBApiClient {

    private $userid;
    private $usersession;  
    
    private static $instance = null;
    public static function instance() {
        if(!isset(self::$instance)) {
            $clientname = CLIENT_NAME;
            $clientversion = CLIENT_VERSION;
            $platformname = PLATFORM_NAME;
            $platformversion = PLATFORM_VERSION;
            $api_url = vB_SITE_ROOT . 'api.php';
            $api_key = vB_API_KEY;

            self::$instance = new VBApiClient($api_url, $clientname, $clientversion, $platformname, $platformversion, $api_key);
        }
        return self::$instance;
    }
    
	private function __construct($site, $clientname, $clientversion, $platformname, $platformversion, $apikey)
	{
		$this->vurl = new vB_vURL();
		$this->vurl->set_option(VURL_HTTPHEADER, array("X-Forwarded-For: " . $_SERVER['REMOTE_ADDR']));
		$this->vurl->set_option(VURL_REFERER, $_SERVER['REQUEST_URI']);
		$this->vurl->set_option(VURL_USERAGENT,$_SERVER['HTTP_USER_AGENT']);
        $this->vurl->set_option(VURL_RETURNTRANSFER, true);
		$this->vurl->set_option(VURL_VALIDSSLONLY, false);
        $this->vurl->set_option(VURL_FOLLOWLOCATION, true);
        $this->vurl->set_option(VURL_MAXREDIRS, 1);
		$this->site = $site;
		$this->apikey = $apikey;

		$this->clientname = $clientname;
        $this->clientversion = $clientversion;
        $this->platformname = $platformname; 
        $this->platformversion = $platformversion;
        
        if( !isset($_SESSION[self::session_namespace() .'api_s']) ||
            !isset($_SESSION[self::session_namespace() .'api_c']) ||
            !isset($_SESSION[self::session_namespace() .'api_v']) ||
            !isset($_SESSION[self::session_namespace() .'secret']) ||
            !isset($_SESSION[self::session_namespace() .'vb_version']) ||
            !isset($_SESSION[self::session_namespace() .'features']) ||
            !isset($_SESSION[self::session_namespace() .'uniqueid'])) {
            $this->api_init();
        } else {
            $this->api_s = $_SESSION[self::session_namespace() .'api_s'];
            $this->api_c = $_SESSION[self::session_namespace() .'api_c'];
            $this->api_v = $_SESSION[self::session_namespace() .'api_v'];
            $this->secret = $_SESSION[self::session_namespace() .'secret'];
            $this->vb_version = $_SESSION[self::session_namespace() .'vb_version'];
            $this->uniqueid = $_SESSION[self::session_namespace() .'uniqueid'];
            $this->features = $_SESSION[self::session_namespace() .'features'];            
        }
	}
    
    private static function session_namespace() {
        return 'vb_api_client_';
    }
	
    private function api_init() {
        $this->uniqueid = $_SESSION[self::session_namespace() .'uniqueid'] = md5(time() . rand());
        $relative_url ="?api_m=api_init&clientname=".urlencode($this->clientname)."&clientversion=".$this->clientversion.
		"&platformname=".$this->platformname."&platformversion=".$this->platformversion."&uniqueid=".$this->uniqueid;
        $json_response = $this->get($relative_url);
		$response	  = json_decode($json_response);
		$this->api_s  = $_SESSION[self::session_namespace() .'api_s'] = @$response->apiaccesstoken;
		$this->api_v  = $_SESSION[self::session_namespace() .'api_v'] = @$response->apiversion;
		$this->api_c  = $_SESSION[self::session_namespace() .'api_c'] = @$response->apiclientid;
		$this->secret = $_SESSION[self::session_namespace() .'secret'] = @$response->secret;
        $this->vb_version = $_SESSION[self::session_namespace() .'vb_version'] = @$response->vbulletinversion;
        $this->features = $_SESSION[self::session_namespace() .'features'] = @$response->features;
    }
    
    public function getBlogsEnabled() {
        return (isset($this->features->blogsenabled) && $this->features->blogsenabled);
    }
    public function getArticlesEnabled() {
        return (isset($this->features->cmsenabled) && $this->features->cmsenabled);
    }
    public function getFacebookEnabled() {
        return (isset($this->features->facebookenabled) && $this->features->facebookenabled);
    }
    
    public function getVBMajorVersion() {
        if(!isset($this->vb_version)) {
            $this->api_init();
        }
        $matches = array();
        if(preg_match("/([0-9]+)\\.([0-9]+)\\.[0-9]+/", $this->vb_version, $matches)) {
            return $matches[1];
        }
        return -1;       
    }
    public function getVBMinorVersion() {
        if(!isset($this->vb_version)) {
            $this->api_init();
        }
        $matches = array();
        if(preg_match("/([0-9]+)\\.([0-9]+)\\.[0-9]+/", $this->vb_version, $matches)) {
            return $matches[2];
        }
        return -1;
    }
    
        
    // Cache identical api calls made during the same request, not across the session.
    private $request_api_cache = array();
    public function call_cached($method, $get, $post) {
        $params = '?' . http_build_query(array_merge($get, $post));
        if(!isset($this->request_api_cache[$method][$params])) {
            $this->request_api_cache[$method][$params] = $this->call($method, $get, $post);
        }
        return $this->request_api_cache[$method][$params];
    }
    
    public function call_cached_session($method, $get, $post, $can_be_this_old = 120) {
        $params = '?' . http_build_query(array_merge($get, $post));
        if(!isset($_SESSION[self::session_namespace() . 'call_cache'][$method][$params]) || (time() - $_SESSION[self::session_namespace() . 'call_cache'][$method][$params]['time']) > $can_be_this_old) {
            $_SESSION[self::session_namespace() . 'call_cache'][$method][$params]['data'] = $this->call_cached($method, $get, $post);
            $_SESSION[self::session_namespace() . 'call_cache'][$method][$params]['time'] = time();
        }
        return $_SESSION[self::session_namespace() . 'call_cache'][$method][$params]['data'];
    }
    
	public function call($method, $get, $post) {
		$start = microtime(true);
        $get['api_m'] = $method;
		$apiparams['api_s'] = $this->api_s;
		$apiparams['api_c'] = $this->api_c;
		$apiparams['api_v'] = 4;
		$apiparams['api_sig'] = $this->sign($get);

		$url = '?' . http_build_query($apiparams, '', '&') . '&' . http_build_query($get, '', '&');
		$response = $this->post($url, $post);
        $json_response = json_decode($response);
        if(isset($json_response->response->errormessage))
        {
            $error_tag = $json_response->response->errormessage[0];
            if( $error_tag == 'invalid_clientid' ||
                $error_tag == 'invalid_accesstoken' ||
                $error_tag == 'invalid_api_signature' 
              ) {
                $this->api_init();
                $response = $this->post($url, $post);
            }
            else if($error_tag == 'nopermission_loggedout' || 
                    $error_tag == 'invalid_sessionhash') {
                $_SESSION['logged_in_to_vb'] = false;
                if($this->getVBMajorVersion() === 4) {
                    if($this->facebook_login($_SESSION['facebook_session']) == 1)
                        $response = $this->post($url, $post);
                } else if(isset($_SESSION['user_credentials'])) {
                    $aux_response = json_decode($this->login($_SESSION['user_credentials']));
                    if(is_array($aux_response->response->errormessage)) {
                        $error_response = $aux_response->response->errormessage[0];
                    } else {
                        $error_response = $aux_response->response->errormessage;
                    }
                    if($error_response == "redirect_login") {
                        $response = $this->post($url, $post);                      
                    }
                }
            }
        }
        $end = microtime(true);
        $this->timeinapi += $end - $start;
		return $response;
	}
    
    public static function getKeysFromResponse($response, $keys, $retarray, $depth = 3) {
        if(!is_array($response) || $depth < 1) {
            return $retarray;
        }
        foreach($response as $key => $value) {
            if(!is_numeric($key) && in_array($key, $keys)) {
                $retarray[$key] = $value;
            } else {
                $retarray = self::getKeysFromResponse($value, $keys, $retarray, --$depth);
            }
        }
        return $retarray;
    }
    
    public function getUserid() {
        if(!isset($this->userid)) {
            if(isset($_SESSION[self::session_namespace() .'userid'])) {
                $this->userid = $_SESSION[self::session_namespace() .'userid'];
            }
        }
        return $this->userid;
    }
    
    public function getUsersession() {
        if(!isset($this->usersession)) {
            if(isset($_SESSION[self::session_namespace() .'usersession'])) {
                $this->usersession = $_SESSION[self::session_namespace() .'usersession'];
            }
        }
        return $this->usersession;
    }

    private $timeinapi = 0;
    public function timeInAPIThisRequest() {
        return $this->timeinapi;
    }

	private function sign($params) {
		ksort($params);
		$signstr = http_build_query($params, '', '&');
		$sign = md5($signstr . $this->api_s . $this->api_c . $this->secret . $this->apikey);
				
		return $sign;	
	}
		
	public function facebook_login($session) {
        $login = $this->call('login_facebook', array(),
        array('signed_request' => $session));
        $aux_response = json_decode($login);
        $error_response = $aux_response->response->errormessage[0];
        if($aux_response->response->errormessage[0] == "redirect_login"){
            $this->userid = $aux_response->session->userid;
            $this->usersession = $aux_response->session->dbsessionhash;
            $_SESSION[self::session_namespace() .'userid'] = $this->userid;
            $_SESSION[self::session_namespace() .'usersession'] = $this->usersession;
            $_SESSION['logged_in_to_vb'] = true;
            return 1;
        } else {
            if($error_response == 'badlogin') return "Username or Password incorrect. Login failed.";
            if($error_response == 'badlogin_strikes' || $error_response == 'strikes' || $error_response == 'badlogin_strikes_passthru') return "Username or Password incorrect. Login failed. After 5 attempts have been used, you are unable to login for 15 minutes.";
        }
        return 0;
	}
    
    public function login($post) {
        $login = $this->call('login_login', array(), $post);
        $aux_response = json_decode($login);
        $error_response = '';
        if(is_array($aux_response->response->errormessage)) {
            $error_response = $aux_response->response->errormessage[0];
        } else {
            $error_response = $aux_response->response->errormessage;
        }
        if($error_response == "redirect_login") {
            $this->userid = $aux_response->session->userid;
            $this->usersession = $aux_response->session->dbsessionhash;
            $_SESSION[self::session_namespace() .'userid'] = $this->userid;
            $_SESSION[self::session_namespace() .'usersession'] = $this->usersession;
            $_SESSION['logged_in_to_vb'] = true;
            $_SESSION['user_credentials'] = $post;
        }
        return $login;
	}
    
    public function logout() {
        $_SESSION['logged_in_to_vb'] = false;
        unset($_SESSION['user_credentials']);
        $logout = $this->call('login_logout', array(), array());
        $aux_response = json_decode($logout);
        $this->userid = 0;
        $this->usersession = $aux_response->session->dbsessionhash;
        $_SESSION[self::session_namespace() .'userid'] = $this->userid;
        $_SESSION[self::session_namespace() .'usersession'] = $this->usersession;
        $this->api_init();
    }
	function register_member($username, $email,$emailconfirm, $password, $passwordconfirm) {
        $this->call_cached("register", array(), array());
        $this->call_cached("register_addmember", array(), array('agree' => true, 'username' => $username, 'email' => $email, 'emailconfirm' => $emailconfirm, 'password' => $password, 'passwordconfirm' => $passwordconfirm));	
	}
	
    public function register_addmember($session, $fbusername, $fbuserid, $username, $password, $email) {
        $post['signed_request'] = $session;
        $post['agree'] = 1;
        $post['username'] = $username;
        $post['email'] = $email;
        $post['emailconfirm'] = $email;
        $post['password'] = $password;
        $post['passwordconfirm'] = $password;
        $post['fbname'] = $fbusername;
        $post['fbuserid'] = $fbuserid;

        $result = $this->call_cached("register_addmember", array(), $post);
        return json_decode($result, true);
    }
	
	function permission_errors($error) {
		switch($error) {
			case 'forumpasswordmissing': return "The forum requires a password to view."; break;
			case 'redirect_postthanks_moderate': return "The new thread has been posted successfully. But it has been put to moderation queue and is waiting moderators to approve."; break;
			case 'invalidid': return "Invalid threadid parameter."; break;
			case 'forumclosed': return "Forum is closed."; break;
			case 'threadclosed': return "Thread is closed."; break;
			case 'session_timed_out_login': return "User account time out, or user not logged"; break;
			case 'nopermission_banned': return "The user has been banned"; break;
			case 'nopermission_banned_infractions': return "The user has been banned because of enough infractions"; break;
			case 'nopermission_loggedin': return "The user has logged in but has no permission to call the method."; break;
			case 'nopermission_loggedout': return "The guest has no permission to call the method. Client may login user and the user may then get the permission."; break;
            case 'emaildisabled': return "the site does not have permissions to send reports. you need to put in contact with your admin."; break;
            case 'redirect_duplicatepost': return "The site don't allow the user duplicate posts many times."; break;
            case 'tooshort': return "The message is too short."; break;
            case 'nosubtype': return "Invalid forumid or threadid parameter."; break;
            case 'duplicate_post': return "Post is a duplicate."; break;
            case 'postfloodcheck': return "You are trying to post too quickly, please wait a few seconds and try again."; break;
			default: return $error;
		}				
	}

	protected function get($relative_url) {
        $this->vurl->set_option(VURL_URL, $this->site . $relative_url);
        $this->vurl->set_option(VURL_POST, false);
        $this->vurl->set_option(VURL_HTTPGET, true);
        
        return $this->vurl->exec($this->site . $relative_url);
	}
	
	protected function post($relative_url, $data) {            
        $strdata = array();
        foreach ($data AS $param => $datum) {
            if(is_array($datum)) {
                foreach($datum as $keyDatum => $datumum) {
                    if($param == 'userfield'){
                        if(is_array($datumum)){
                            foreach($datumum as $multiple){
                                $strdata[] = $param . "[" . $keyDatum . "][]=" . urlencode($multiple);
                            }
                        }else
                           $strdata[] = $param . "[" . $keyDatum . "]=" . urlencode($datumum);
                    }else if($param == 'optionnumber' && is_array($data)){
                        foreach($datum as $option => $value){
                            $strdata[] = $param . "[" . $option . "]=" . $value;
                        }
                    }else
                        $strdata[] = "$param=" . urlencode($datumum);
                }
            } else {
                $strdata[] = "$param=" . urlencode($datum);
            }
        }
        $data = implode('&', $strdata);
        unset($strdata);
        $this->vurl->set_option(VURL_URL, $this->site . $relative_url);
        $this->vurl->set_option(VURL_POST, true);
        $this->vurl->set_option(VURL_HTTPGET, false);
        $this->vurl->set_option(VURL_POSTFIELDS, $data);
        $result = $this->vurl->exec();
        $this->vurl->set_option(VURL_POSTFIELDS, "");
        return $result;
	}	
}
?>
